using AddressBookApi.DataStore;
using AddressBookApi.Models.Dto;
using Microsoft.AspNetCore.Mvc;

namespace AddressBookApi.Controllers;

[ApiController]
[Route("api/AddressBookApi")]
public class AddressBookApiController : ControllerBase
{
    [HttpGet]
    public ActionResult<IEnumerable<AddressDto>> GetAddresses()
    {
        return Ok(AddressStore.addressList);
    }

    [HttpGet("{id:int}", Name = "GetAddressById")]
    [ProducesResponseType(200)]
    [ProducesResponseType(400)]
    [ProducesResponseType(404)]
    public ActionResult<AddressDto> GetAddress(int id)
    {
        if (id == 0)
        {
            return BadRequest();
        }

        var address = AddressStore.addressList.FirstOrDefault((u => u.Id == id));
        if (address == null)
        {
            return NotFound();
        }

        return Ok(address);
    }

    [HttpGet("{name}", Name = "GetAddressByName")]
    [ProducesResponseType(200)]
    [ProducesResponseType(400)]
    [ProducesResponseType(404)]
    public ActionResult<AddressDto> GetAddress(string name)
    {
        if (name == "")
        {
            return BadRequest();
        }

        var address = AddressStore.addressList.FirstOrDefault(u => u.Name == name);
        if (address == null)
        {
            return NotFound();
        }

        return Ok(address);
    }

    [HttpPost]
    [ProducesResponseType(201)]
    [ProducesResponseType(400)]
    public ActionResult<AddressDto> AddAddress([FromBody] AddressDto newAddressDto)
    {
        if (AddressStore.addressList.FirstOrDefault(u=>String.Equals(u.Name, newAddressDto.Name, StringComparison.CurrentCultureIgnoreCase)) != null)
        {
            ModelState.AddModelError("Address Error 1", "Addressee Already Exists!");
            return BadRequest(ModelState);
        }
        if (newAddressDto.Id > 0)
        {
            return StatusCode(StatusCodes.Status400BadRequest);
        }

        if (newAddressDto.Name == "")
        {
            return StatusCode(StatusCodes.Status400BadRequest);
        }

        if (newAddressDto.Street == "")
        {
            return StatusCode(StatusCodes.Status400BadRequest);
        }

        if (newAddressDto.City == "")
        {
            return StatusCode(StatusCodes.Status400BadRequest);
        }

        if (newAddressDto.PostCode == "")
        {
            return StatusCode(StatusCodes.Status400BadRequest);
        }

        newAddressDto.Id = AddressStore.addressList.MaxBy(u => u.Id)!.Id + 1;
        AddressStore.addressList.Add(newAddressDto);
        return CreatedAtRoute("GetAddressById", new { id = newAddressDto.Id }, newAddressDto);
    }

    [HttpDelete("{id:int}", Name = "DeleteAddress")]
    [ProducesResponseType(204)]
    [ProducesResponseType(400)]
    [ProducesResponseType(404)]
    public IActionResult DeleteAddress(int id)
    {
        if (id == 0)
        {
            return BadRequest();
        }

        var address = AddressStore.addressList.FirstOrDefault(u => u.Id == id);
        if (address == null)
        {
            return NotFound();
        }
        AddressStore.addressList.Remove(address);
        return NoContent();
    }
}